? my $app = shift;

<div id="help" style="display:none">
  <h1>Alice Help</h1>
  <div id="helpclose">close</div>
  <div id="topics">
    <dl>
      <h2>Commands</h2>
      <? for my $command (@{$app->commands->handlers }) { ?>
        <? next unless $command->{eg} and $command->{desc}; ?>
        <dt><?= $command->{eg} ?></dt>
        <dd><?= $command->{desc} ?></dd>
      <? } ?>
    </dl>
    <dl>
      <h2>Shortcuts</h2>
      <dt title="Command + K">⌘K</dt>
      <dd>Clear current tab</dd>
      <dt title="Command + B">⌘B</dt>
      <dd>Previous tab</dd>
      <dt title="Command + F">⌘F</dt>
      <dd>Next tab</dd>
      <dt title="Option + Up Arrow">⌥↑</dt>
      <dd>Cycle through previous message</dd>
      <dt title="Option + Down Arrow">⌥↓</dt>
      <dd>Cycle to next message</dd>
      <dt title="Option + Return">⌥↵</dt>
      <dd>Insert a newline</dd>
      <dt title="Command + M" >⌘⇧M</dt>
      <dd>Mark all tabs as read</dd>
      <dt title="Command + J">⌘⇧J</dt>
      <dd>Jump to bottom of tab</dd>
      <dt title="Cmomand + Shift + K">⌘⇧K</dt>
      <dd>Show timestamps and nicks</dd>
    </dl>
  </div>
</div>
