? my ($app, $msg) = @_;
? my $avatar = ($msg->{avatar} and $app->config->{avatars} eq "show") ? $msg->{avatar} : "";
<li class="message <?= $msg->{highlight} ? "highlight " : "" ?><?= $avatar ? "avatar " : "" ?><?= $msg->{nick} ?><?= $msg->{self} ? " self" : "" ?><?= $msg->{consecutive} ? " consecutive" : "" ?><?= $msg->{monospaced} ? " monospace" : ""?>" id="<?= $msg->{msgid} ?>">
    <div class="left">
      <a href="javascript:alice.connection.requestWindow('<?= $msg->{nick} ?>','<?= $msg->{window}{id} ?>')" class="nick">
        <? if ($avatar) { ?>
          <img src="<?= $avatar ?>" alt="<?= $msg->{nick} ?>" />
          <span class="nickhint"><?= $msg->{nick} ?></span>
        <? } else { ?>
          <?= $msg->{nick} ?>
        <?}?>
      </a>
    </div>
    <div class="msg"><?= $msg->{html} ?></div>
    <? if (!$msg->{consecutive} and $msg->{timestamp}) { ?>
    <div class="timehint"><?= $msg->{timestamp} ?></div>
    <? } ?>
</li>
